# $Id: Import.pm,v 0.2 1999/07/28 06:07:27 fil Exp fil $
# This hack^Wpackage is provided AS IS, with no warranty etc etc
# Copyright 1998, 1999, Philip Gwyn.  All rights reserved.
# Licensed under the Artistic licence

package Import;
use strict;
use vars qw(@EXPORT $VERSION);

$VERSION='0.2';

BEGIN {
    require Exporter;
    @EXPORT = qw(AUTOLOAD);
}

sub import 
{
    my $pkg = shift;
    my $callpkg = caller;

    # Export symbols, but not by accident of inheritance.
    # (got this idea from AutoLoad)
    Exporter::export $pkg, $callpkg, @_ if $pkg eq 'Import';

}

## This is the meat of the module
sub AUTOLOAD
{
    use vars qw($AUTOLOAD);

    ## what subroutine are we looking for
    my $q=rindex($AUTOLOAD, '::');
    my $sub=substr($AUTOLOAD, $q+2);
    my $package=substr($AUTOLOAD, 0, $q);

    ## Dig into the symtable to get it
    my $code=_get_code($package, $sub);

    ## jump to that routine
    if($code)
    {
        _get_table($package)->{$sub}=\&$code;
        goto &$code;
    }
    ## Well, maybe not
    require Carp;
    Carp::croak("Undefined subroutine $AUTOLOAD called");
}

## Recursively dig around in the symtable to find a function of a given name
#
sub _get_code
{
    my($package, $sub)=@_;
    my $table=_get_table($package);
    my $code;

    if(exists $table->{$sub})   # possible...
    {
        $code=$table->{$sub};   # so we have a entry
        if(defined &$code)      # but is it a coderef?
        {
            return $code;       # YES!
        }
    }
    return unless $table->{ISA};    # no more places to look

    # ok, now look in more packages
    # fortunately, @ISA is searched depth-first or this would be much hairier
    foreach (@{$table->{ISA}})  
    {
        next if $_ eq __PACKAGE__;      # DON'T look in ours
        $code=_get_code($_, $sub);      # recurse
        return $code if defined $code;  
    }
}

## Returns a symbol table
#
sub _get_table
{
    my($package)=@_;
    my $table=$::{'main::'};
    foreach my $b (split '::', $package)
    {   $table=$table->{$b.'::'};}
    return $table;
}
1;

__END__
$Log: Import.pm,v $
Revision 0.2  1999/07/28 06:07:27  fil
Make an alias

=head1 NAME

Import - @ISA for regular subroutines

=head1 SYNOPSIS

  package Bar;
  sub hello
  {
      print "Hello world\n";
  }

  package main;
  use Import;
  @main::ISA=qw(Bar);
  hello();

=head1 DESCRIPTION

This is the package your mother warned you about.  Beware meeting this
package in dark alleys and undocumented code.  Pheare it's use!

Perl reserves the the @ISA array for method inheritance.  However, it
only works in OO constructs.

    $self->hello();     # object method
    main->hello();      # class method
 
With Import, regular subroutines will also be "inherited".  We do this with
an AUTOLOAD function that digs through the symbol table and @ISA chain
looking for the apropriate entry.

=head1 JUSTIFICATION

Some modules don't export the functions you would want them to.  This was
the case of L<HTML::Mason::Commands> before 0.4.  Rather then modify the
distributed code, I wrote this module to "forcibly" import functions. This
approche has the advantage that you don't have to list all the functions you
want at the begining of the module; the functions are imported as needed.

Another reason I'm doing this is, well, because I *can*.  :)

=head1 WARNING

Both Tom Christiansen and Randal Schwartz have expressed dismay at this
module.  Randal pointed out that exporting AUTOLOAD as this module does is a
bad idea.  YMMV.

=head1 AUTHOR

Philip Gwyn, fil@artware.qc.ca

=head1 SEE ALSO

perl(1).

=cut
